# Approaching Terminus - beeps.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import splat
import splat.data
import splat.gen
import splat.filters
import splat.interpol
import tools

def pulses(master, start, gen, end, period, ratio, args, levels):
    gen.frag = splat.data.Fragment(rate=master.rate, channels=master.channels)
    gen.levels = levels
    t_on = period * ratio
    cur = start
    while cur < end:
        gen.run(cur, (cur + t_on), *args)
        cur += period
    master.mix(gen.frag)

def main(argv):
    f_out = argv[1]
    fade = lambda t: [(splat.filters.linear_fade, (t,))]
    sine = splat.gen.SineGenerator(filters=fade(0.015))
    triangle = splat.gen.TriangleGenerator(filters=fade(0.01))
    square = splat.gen.SquareGenerator(filters=fade(0.001))
    beeps = splat.data.Fragment()

    lvl1_pts = [(0.0, 0.0), (2.0, 0.0, 0.0), (3.0, -18.0), (4.0, -12.0, 0.0),
                (28.0, -100.0)]
    lvl1 = splat.interpol.spline(lvl1_pts)

    ph4_pts = [(3.7, 0.0)]
    for x in range(3):
        x0 = ph4_pts[-1][0]
        ph4_pts += [(x0 + 5.0, 0.1), (x0 + 6.0, -0.1), (x0 + 8.0, 0.0)]
    ph4 = splat.interpol.spline(ph4_pts)
    lvl4 = splat.data.Fragment(channels=1)
    f4 = [(ph4.start, 1.0), ((ph4.end - ph4.start) / 2, 3.0), (ph4.end, 10.0)]
    fm4 = splat.interpol.freqmod(f4)
    fm_gen4 = splat.gen.SineGenerator(frag=lvl4)
    fm_gen4.run(fm4.start, fm4.end, fm4.f0, fm4.value, levels=18.0)
    env4_pts = [(0.0, -23.0), (fm4.start + 0.5, -21.0, 0.0),
                (fm4.end - 3.0, -27.0), (fm4.end, -100.0)]
    env4 = splat.interpol.spline(env4_pts)
    lvl4.offset(env4.value)

    layers = [
        (lvl1.start, (sine, lvl1.end, 0.12, 0.3, (880.0,), lvl1.value)),
        (1.3, (triangle, 23.0, 0.87, 0.43, (325.0,), -3.0)),
        (1.3, (triangle, 23.0, 0.86, 0.43, (327.0,), -3.0)),
        (ph4.start, (square, ph4.end, 0.25, 0.8, (1267.0, ph4.value), lvl4)),
        ]

    for start, args in layers:
        pulses(beeps, start, *args)

    duration = max(lvl1.end, lvl4.duration, ph4.end)
    lvl = [(0.0, -100.0), (2.0, 0.0, 0.0),
           (duration - 5.0, -3.0), (duration, -100.0)]

    tools.rotamp(beeps, 1 / 0.3, 12.0, 3)
    beeps.normalize()
    beeps.save(f_out)

    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
