# Approaching Terminus - breathe.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import splat
import splat.data
import splat.gen
import splat.interpol
import splat.plot

audio_input = "input/breathing.flac"
duration = 75.0
levels = [(0.0, -100.0), (2.0, -30.0), (3.0, -20.0), (30.0, 0.0, 0.0),
          (duration - 15.0, 0.0), (duration, -100.0)]

def main(argv):
    f_out = argv[1]
    if (len(argv) > 2) and (argv[2] == 'plot'):
        splat.plot.spline(splat.interpol.spline(levels))
        return True
    frag_in = splat.data.Fragment.open(audio_input)
    start = frag_in.duration - duration
    frag = splat.data.Fragment()
    frag.mix(frag_in, start=start, duration=duration)
    frag.amp(splat.interpol.spline(levels).value)
    frag.save(f_out, end=frag.s2n(duration))
    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
