# Approaching Terminus - continuum.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import random
import splat
import splat.data
import splat.gen
import splat.interpol
import splat.plot
import splat.scales
import splat.plot
import tools
import heartbeat

duration = 95.0
bass_levels = [(0.0, -100.0), (12.0, 0.0, 0.0),
               (duration - 8.0, 0.0), (duration, -100.0)]
bass_notes = [['C-3', 'D-3', 'D-3', 'F-3', 'A-3'],
              ['E-2', 'E-2', 'G-2', 'B-2', 'D-2']]
melody_levels = [(0.0, -100.0), (20.0, -24.0, 0.0),
                 (duration - 10.0, -32.0), (duration, -100.0)]
melody_notes = [['Db1', 'Eb1', 'Eb1', 'G1', 'Bb1'],
                [['F1', 'F1', 'Ab1', 'C1', 'Eb2'], ['Ab0', 'Bb0', 'Eb1']]]

def bass(gen, scale, period, notes, levels):
    bass = splat.data.Fragment()
    gen.frag = bass
    gen.ot_decexp(1.0)
    for x in range(int(duration / period)):
        t = x * period
        for n in notes:
            gen.run(t, (t + period), scale[random.choice(n)])
    bass.amp(splat.interpol.spline(levels).value)
    return bass

def melody(gen, scale, period, notes, levels):
    melody1 = splat.data.Fragment()
    gen.frag = melody1
    gen.ot_decexp(0.4)
    part1 = []
    for x in range(int(duration / period)):
        part1.append((x * period, scale[random.choice(notes[0])], 0.0))
    fm = splat.interpol.freqmod(part1)
    gen.run(fm.start, fm.end, fm.f0, fm.value)
    tools.rotamp(melody1, period * 3, 12.0, 3)

    melody2 = splat.data.Fragment()
    gen.frag = melody2
    period2 = period * 5 / 3
    part2 = []
    gen.overtones = [(1.0, 0.0, -6.0), (2.0, 0.0, -3.0), (4.0, -12.0, 0.0),
                     (8.5, 0.0, -24.0), (17.0, 0.0, -50.0)]
    for x in range(int(duration / period2)):
        i = (x / 3) % 2
        part2.append((x * period2, scale[random.choice(notes[1][i])], 0.0))
    fm = splat.interpol.freqmod(part2)
    gen.run(fm.start, fm.end, fm.f0, fm.value)
    tools.rotamp(melody2, period2 * 2, 12.0, 2.5)

    melody = splat.data.Fragment()
    melody.mix(melody1)
    melody.mix(melody2)
    melody.amp(splat.interpol.spline(levels).value)
    delays = splat.filters.reverb_delays(rate=500, dec=0.5, length=1.0)
    splat.filters.reverb(melody, delays)

    return melody

def main(argv):
    f_out = argv[1]
    period = splat.data.Fragment.open(heartbeat.audio_input).duration
    frag = splat.data.Fragment()
    gen = splat.gen.OvertonesGenerator()
    gen.filters = splat.filters.FilterChain(
        [(splat.filters.linear_fade, (period / 4.7,))])
    scale = splat.scales.LogScale(fund=440.0)
    frag.mix(bass(gen, scale, period, bass_notes, bass_levels))
    frag.mix(melody(gen, scale, period, melody_notes, melody_levels))
    frag.normalize()
    frag.save(f_out)
    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
