# Approaching Terminus - low_modulation.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import math
import random
import splat
import splat.data
import splat.gen
import splat.interpol
import splat.plot
import splat.scales
import splat.plot
import heartbeat

duration = 76.0
levels = [(0.0, -100.0), (12.0, 0.0, 0.0),
          (duration - 6.0, 0.0), (duration, -100.0)]

def main(argv):
    f_out = argv[1]
    frag = splat.data.Fragment()
    gen = splat.gen.TriangleGenerator(frag=frag)
    freq = splat.scales.LogScale(fund=440.0)['B-3']
    period = splat.data.Fragment.open(heartbeat.audio_input).duration * 16 / 5
    k = math.pow(2, (1 / 48.0)) - 1
    pts = []
    for x in range(int(duration / period) + 1):
        t = x * period
        pts.append((t, freq * random.uniform((1 - k), (1 + k)), 0.0))
    f_spline = splat.interpol.spline(pts)
    gen.run(0.0, f_spline.end, freq=f_spline.value)
    splat.gen.SineGenerator(frag=frag).run(
        0.0, f_spline.end, freq=(freq * 1.5), levels=-3.0)
    frag.amp(splat.interpol.spline(levels).value)
    frag.normalize()
    frag.save(f_out)
    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
