# Approaching Terminus - terminus.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import splat
import splat.data
import splat.gen
import splat.filters
import splat.interpol
import splat.plot

jump = 118.0

sequence = [
    # (file_name, offset, levels),
    ('breathe.saf', 0.5, 6.0),
    ('low_modulation.saf', 3.0, (-7.0, -4.0)),
    ('beeps.saf', 23.0, None),
    ('continuum.saf', 41.0, (-5.0, -3.0)),
    ('heartbeat.saf', 51.0, 2.5),
    ('parameters.saf', jump - 25.0, -12.0),
    ('countdown.saf', jump - 10.1, -12.0),
    ('approaching.saf', jump, -12.0),
    ('landing.saf', jump + 10, None),
    ('descent.saf', jump + 14, 0.0),
    ('steam.saf', jump + 78, 4.0),
    ('wind.saf', jump + 84, 3.0),
    ]

def main(argv):
    f_out = argv[1]
    master = splat.data.Fragment()
    for fname, ofst, ml in sequence:
        master.mix(splat.data.Fragment.open(fname), offset=ofst, levels=ml)
    master.normalize()
    split = splat.data.Fragment()
    split.mix(master, duration=jump)
    jump_gap = splat.data.Fragment.open('jump.saf')
    split.mix(jump_gap, offset=jump)
    split.mix(master, start=jump, offset=(jump + jump_gap.duration))
    split.save(f_out)
    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
