# Approaching Terminus - wind.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import splat
import splat.data
import splat.gen
import splat.filters

audio_input = "input/wind.flac"
duration = 60.0
levels = [(0.0, -100.0), (4.0, 0.0, 0.0), (duration / 2, -10.0),
          (duration - 10.0, -20.0), (duration, -100.0)]

def main(argv):
    f_out = argv[1]
    frag = splat.data.Fragment.open(audio_input)
    frag.resize(duration)
    frag.amp(splat.interpol.spline(levels).value)
    delays = splat.filters.reverb_delays(rate=40, length=3.0, dec=0.05)
    splat.filters.reverb(frag, delays)
    frag.normalize()
    frag.save(f_out, end=frag.s2n(duration))
    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
